'use strict';

var React = require('react');
var createProxy = require('react-proxy').default;

function setFlag(obj, key) {
  try {
    Object.defineProperty(obj, key, {
      configurable: true,
      enumerable: false,
      value: true
    });
  } catch (err) {}
}

var proxies = {};
var warnedAboutTypes = {};
function resolveType(type) {
  if (!type || typeof type === 'string') {
    return type;
  }

  if (!Object.hasOwnProperty.call(type, '__source') || !type.__source || !type.__source.fileName || !type.__source.localName) {
    setFlag(type, '__noSourceFound');
    return type;
  }

  var source = type.__source;
  var id = source.fileName + '#' + source.localName;

  if (type.hasOwnProperty('__noSourceFound')) {
    if (!warnedAboutTypes[id]) {
      warnedAboutTypes[id] = true;
      console.error('React Hot Loader: ' + source.localName + ' from ' + source.fileName + ' will not ' + 'hot reload correctly because it contains an imperative call like ' + ('ReactDOM.render() in the same file. Split ' + source.localName + ' into a ') + 'separate file for hot reloading to work.');
    }
    return type;
  }

  if (!proxies[id]) {
    proxies[id] = createProxy(type);
  } else if (!type.hasOwnProperty('__hasBeenUsedForProxy')) {
    proxies[id].update(type);
  }
  setFlag(type, '__hasBeenUsedForProxy');

  return proxies[id].get();
}

if (React.createElement.isPatchedByReactHotLoader) {
  throw new Error('Cannot patch React twice.');
}

var createElement = React.createElement;
function patchedCreateElement(type) {
  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  return createElement.apply(undefined, [resolveType(type)].concat(args));
}
patchedCreateElement.isPatchedByReactHotLoader = true;
React.createElement = patchedCreateElement;